# FILE: makerules.tcl
#
# This file controls the application 'pimake' by defining rules which
# describe how to build the applications and/or extensions produced by
# the source code in this directory.
#
# Verify that this script is being sourced by pimake
if {[llength [info commands MakeRule]] == 0} {
    error "'[info script]' must be evaluated by pimake"
}
#
########################################################################

set commonObjects {
	fileio
	mesh
	ptsearch
	vecfile
	vector
}

set mmdispObjects {
	display
	mmdinit
	mmdisp
}

set allObjects [concat avftoovf avftovio $mmdispObjects $commonObjects]

MakeRule Define {
    -targets            build
    -dependencies       [concat configure tclIndex \
			        [Platform Executables {avftoovf avftovio} ] \
			        [Platform Specific appindex.tcl]]
}

MakeRule Define {
    -targets            configure
    -dependencies       [Platform Name]
}

MakeRule Define {
    -targets            [Platform Name]
    -dependencies       {}
    -script             {MakeDirectory [Platform Name]}
}

MakeRule Define {
    -targets            tclIndex
    -dependencies       [glob -nocomplain *.tcl]
    -script             {
                         MakeTclIndex .
                        }
}

# Could use better support for this:
MakeRule Define {
    -targets            [Platform Specific appindex.tcl]
    -dependencies       [Platform Executables mmdisp]
    -script             {puts "Updating [Platform Specific appindex.tcl]"
                        set f [open [Platform Specific appindex.tcl] w]
                        puts $f [format {
Oc_Application Define {
    -name               mmDisp
    -version            1.0.0.0.0.0
    -machine            %s
    -file               "%s"
}
                        } [Platform Name] \
                                [file tail [Platform Executables mmdisp]]]
                        close $f
                        }
}

MakeRule Define {
    -targets            [Platform Executables mmdisp]
    -dependencies       [concat [Platform Objects \
			        [concat $mmdispObjects $commonObjects]] \
			        [Platform StaticLibraries {nb oc} ]]
    -script		[format {
			    Platform Link -obj {%s} \
			            -lib {nb oc tk} \
			            -sub WINDOWS -out mmdisp
			} [concat $mmdispObjects $commonObjects]]
}

MakeRule Define {
    -targets            [Platform Executables avftoovf]
    -dependencies       [concat [Platform Objects \
			        [concat avftoovf $commonObjects]] \
			        [Platform StaticLibraries {nb oc} ]]
    -script		[format {
			    Platform Link -obj {%s} \
			            -lib {nb oc tk} \
			            -sub WINDOWS -out avftoovf
			} [concat avftoovf $commonObjects]]
}

MakeRule Define {
    -targets            [Platform Executables avftovio]
    -dependencies       [concat [Platform Objects \
			        [concat avftovio $commonObjects]] \
			        [Platform StaticLibraries {nb oc} ]]
    -script		[format {
			    Platform Link -obj {%s} \
			            -lib {nb oc tk} \
			            -sub WINDOWS -out avftovio
			} [concat avftovio $commonObjects]]
}

foreach o $allObjects { 
    MakeRule Define {
        -targets	[Platform Objects $o]
        -dependencies	[concat [list [Platform Name]] \
			        [[CSourceFile New _ $o.cc] Dependencies]]
        -script		[format {Platform Compile C++ -opt 1 \
			        -inc [[CSourceFile New _ %s.cc] DepPath] \
			        -src %s.cc -out %s
			} $o $o $o]
    }
}

MakeRule Define {
    -targets            distclean
    -dependencies       clean
    -script             {DeleteFiles [Platform Name]}
}
 
MakeRule Define {
    -targets            clean
    -dependencies       mostlyclean
}
 
MakeRule Define {
    -targets            mostlyclean
    -dependencies       objclean
    -script             {eval DeleteFiles [concat tclIndex \
			        [Platform Specific appindex.tcl] [Platform \
			        Executables {avftoovf avftovio mmdisp} ]]}
}
 
MakeRule Define {
    -targets            objclean
    -dependencies       {}
    -script             [format {
			    eval DeleteFiles [Platform Objects {%s}]
			} $allObjects]
}

unset allObjects
unset mmdispObjects
unset commonObjects
