# Entry graph canvas sub-widget
#
# debugging enable/disable/level

em_class GraphBox {
    const public variable winpath
    const public variable exit_button = 1  ;# Set to 0 for no exit button
    public variable curve_width = 2  ;# Width of curve plot lines
    const public variable outer_frame_options = \
	    "-bd 4 -relief ridge -background #0000cd"
    # Note: #0000cd is MediumBlue
    const public variable default_height = 320
    const public variable default_width = 320
    const public variable canvas_frame_options = \
	    "-bd 0 -relief ridge -background #0000cd"
    const public variable canvas_options = \
	    "-bd 0 -relief flat -background white"
    const public variable button_frame_options = \
	    "-bd 0 -relief flat -background black"
    const public variable graphsize = 0.8  ;# % of canvas to draw graph in.
    const public variable axescolor = black
    const public variable axeswidth = 2
    const public variable button_color = #a0a0a0
    const public variable button_active_color = #d0d0d0

    const private variable graph  ;# Graph canvas
    const private variable button_frame

    private variable xlabel = ""
    private variable ylabel = ""
    public variable autodraw = 0

    private variable curvecount = 0
    private variable xorg    = 0
    private variable yorg    = 0
    private variable xscale  = 1
    private variable yscale  = 1
    private variable xmargin = 0
    private variable ymargin = 0
    private variable xgraphmin = 0
    private variable xgraphmax = 1
    private variable ygraphmin = 0
    private variable ygraphmax = 1

    private variable color           ;# Color array

    private variable id              ;# Name -> id mapping array
    private variable curve           ;# Curve data array
    ## Each curve has an entry of the following form:
    ##       curve($curveid,name)    = id -> Name mapping
    ##       curve($curveid,color)   = Drawing color
    ##       curve($curveid,pts)     = Data point string of "x y" pairs
    ##       curve($curveid,xmin)    = Minimum value of x among data points
    ##       curve($curveid,xmax)    = Maximum value of x among data points
    ##       curve($curveid,ymin)    = Minimum value of y among data points
    ##       curve($curveid,ymax)    = Maximum value of y among data points
    ##       curve($curveid,ptsdisp) = Display point string of "x y" pairs
    ##       curve($curveid,scalevalid) = True if ptsdisp is properly
    ##                                 scaled for current graph.
    ##       curve($curveid,hidden)  = Controls whether or not curve is drawn
    ##       curve($curveid,button)  = Window path of associated button

    public variable callback = em_nop
    public variable delete_callback_arg = {}

    # Event handling support
    private variable bindtags

    constructor {basewinpath args} {
	# Don't add '.' suffix if there already is one.
	# (This happens chiefly when basewinpath is exactly '.')
	if {![string match {.} \
		[string index $basewinpath \
		[expr [string length $basewinpath]-1]]]} {
	    append basewinpath {.}
	}
	eval $this configure -winpath ${basewinpath}w$this $args

	# Temporarily disable callbacks until construction is complete
	set callback_keep $callback ;	set callback em_nop

	# Pack entire subwidget into a subframe.  This makes
	# housekeeping easier, since the only bindings on
	# $winpath will be ones added from inside the class,
	# and also all subwindows can be destroyed by simply
	# destroying $winpath.
	eval frame $winpath $outer_frame_options
	set graph_frame [eval frame $winpath.canvasframe \
		$canvas_frame_options]
	set graph [eval canvas $graph_frame.graph \
		-width $default_width -height $default_height \
		$canvas_options]
	set button_frame \
		[eval frame $winpath.buttonframe $button_frame_options]
	set btn_rescale [button $button_frame.btn_rescale -text Rescale \
	        -background $button_color \
		-activebackground $button_active_color \
		-command "$this Rescale"]
	set btn_reset [button $button_frame.btn_reset -text Reset \
	        -background $button_color \
		-activebackground $button_active_color \
		-command "$this ResetData"]
	pack $btn_rescale -side top    -anchor n -fill x
	pack $btn_reset   -side top    -anchor n -fill x
	
	if {$exit_button} {
	    set btn_exit   [button $button_frame.btn_exit -text Exit \
		    -background $button_color \
		    -activebackground $button_active_color \
		    -command "$this delete"]
	    pack $btn_exit   -side bottom -anchor s -fill x
	}

	pack $graph -side left -anchor w -fill both -expand 1
	pack $button_frame -side right -anchor ne -fill y -expand 0
	pack $graph_frame -side left -anchor w -fill both -expand 1 
	
	# Initialize color array
	set color [em_array new]
	$color set [list 0 #f00  \
                         1 #0f0  \
	                 2 #00f  \
                         3 #800  \
                         4 #a0a  \
                         5 #0aa  \
                         6 #000  \
                         7 #660  \
                         8 #008  \
                         9 #080   ]
	$color set [list count 10]
 
	# Allocate id and curve arrays
	set id [em_array new]
	set curve [em_array new]

	# Widget bindings
	bind $winpath <Destroy> "$this WinDestroy %W"
	bind $graph <Configure> "$this ResizeCanvas %w %h"

	# Initialize scaling and axes
	$this Rescale

	# Re-enable callbacks
	set callback $callback_keep

        # Event generation support
        set bindtags [list $this $class]
    }
    method ResizeCanvas { width height } { graph } {
	update idletasks  ;# Apparently need this to keep from generating
	## and infinite <Configure> loop
	set border [expr 2*[$graph cget -borderwidth] \
		+2*[$graph cget -highlightthickness]]
	# Note: The <Configure> resize request *includes* both
	#       -borderwidth and -highlightthickness!

	set newwidth [expr $width-$border]
	set newheight [expr $height-$border]
	if {[$graph cget -width]!=$newwidth || \
		[$graph cget -height]!=$newheight} {
	    set confbind [bind $graph <Configure>]
	    bind $graph <Configure> {}  ;# Temporarily disable
	    $graph configure -width $newwidth -height $newheight
	    $this Rescale 0
	    $this DrawCurves
	    bind $graph <Configure> $confbind ;# Reenable
	}
    }
    method DeleteCurve { curvename } {
        if {![$id exists $curvename]} { return } ;# No such curve
	set curveid [$id lookup $curvename]
	destroy [$curve lookup $curveid,button]
	$graph delete tag$curveid
	foreach i [$curve names $curveid,*] {
	    $curve unset $i
	}
	$id unset $curvename
    }
    method ResetGraph {} {
	# Deletes all data and curves
	foreach name [$id names] {
	    $this DeleteCurve $name
	}
    }
    method ResetData {} {
	# Deletes all data, but leaves curves
	foreach name [$id names] {
	    set curveid [$id lookup $name]
	    $curve set [list $curveid,pts {} $curveid,ptsdisp {}]
	    catch { $curve unset $curveid,xmin }
	    catch { $curve unset $curveid,xmax }
	    catch { $curve unset $curveid,ymin }
	    catch { $curve unset $curveid,xmax }
	}
    }
    method NewCurve { curvename } { curvecount id curve color \
	    button_frame button_color button_active_color} {
        if {[$id exists $curvename]} {
	    error "Curve $curvename already exists"
	}
	set curveid $curvecount
	incr curvecount
	set curvecolor [$color lookup [expr $curveid % [$color lookup count]]]
	$id set [list $curvename $curveid]
	$curve set [list                     \
		$curveid,name    $curvename  \
		$curveid,color   $curvecolor \
		$curveid,pts      {}         \
		$curveid,ptsdisp  {}         \
		$curveid,scalevalid 1        \
		$curveid,hidden   0           ]
	catch { $curve unset $curveid,xmin }            ;# Safety
	catch { $curve unset $curveid,xmax }
	catch { $curve unset $curveid,ymin }
	catch { $curve unset $curveid,xmax }

	# Add button
	set btn [button $button_frame.b$curveid \
		-text $curvename \
		-foreground $curvecolor -activeforeground $curvecolor \
	        -background $button_color \
		-activebackground $button_active_color \
		-command "$this ToggleCurveDisplayState $curveid"]
	pack $btn -side top -anchor n -fill x
	$curve set [list $curveid,button $btn]

	return $curveid
    }
    method NewCurves { args } {
	foreach curvename $args { $this NewCurve $curvename}
    }
    method GetCurveId { curvename } { id } {
	# Returns empty string if curvename is invalid
	if {[$id exists $curvename]} {
	    return [$id lookup $curvename]
	}
	return
    }
    method ToggleCurveDisplayState { curveid } { curve graph } {
	if {![$curve exists $curveid,hidden]} {
	    error "Curve $curveid doesn't exist"
	}
	set state [$curve lookup $curveid,hidden]
	set btn [$curve lookup $curveid,button]
	if {$state} {
	    # Toggle display on (hidden off)
	    $curve set [list $curveid,hidden 0]
	    $this PlotCurve $curveid
	    set color [$curve lookup $curveid,color]
	    $btn configure -foreground $color -activeforeground $color
	} else {
	    # Toggle display off (hidden on)
	    $curve set [list $curveid,hidden 1]
	    $graph delete tag$curveid
	    set color [$graph cget -background]
	    $btn configure -foreground $color -activeforeground $color
	}
    }
    private method AddOnePoint { curveid x y } \
	    { curve xorg yorg xscale yscale xmargin ymargin } {
	# Assumes curveid is valid
	$curve append $curveid,pts " $x $y"

	# Data point info
	set extrema {}
	if {$x<[$curve lookup $curveid,xmin]} {
	    lappend extrema $curveid,xmin $x
	} elseif {$x>[$curve lookup $curveid,xmax]} {
	    lappend extrema $curveid,xmax $x
	}
	if {$y<[$curve lookup $curveid,ymin]} {
	    lappend extrema $curveid,ymin $y
	} elseif {$y>[$curve lookup $curveid,ymax]} {
	    lappend extrema $curveid,ymax $y
	}
	if { [llength exterma] > 0 } { $curve set $extrema }
	# Display point info
	set xdisp [expr ($x-$xorg)*$xscale+$xmargin]
	set ydisp [expr ($y-$yorg)*$yscale+$ymargin]
	$curve append $curveid,ptsdisp " $xdisp $ydisp"
	return
    }
    method AddPoints { curvename args } \
	    { id curve autodraw xgraphmin xgraphmax ygraphmin ygraphmax } {
	if {[llength $args] % 2} {
	    error "List argument must have even number of elements\n\n\
		    usage:\n\t\$instance AddPoints curvename list\n\n\
		    Invalid call:\n\t$this AddPoints $curvename $args"
	}
	if {![$id exists $curvename]} {
	    $this NewCurve $curvename
	}
	if {[llength $args]<2} { return }
	set curveid [$id lookup $curvename]
	if {![$curve exists $curveid,xmin]} {
	    # Initial point 
	    set x [lindex $args 0]
	    set y [lindex $args 1]
	    $curve set [list  \
		    $curveid,xmin $x   \
		    $curveid,xmax $x   \
		    $curveid,ymin $y   \
		    $curveid,ymax $y]
	}
	set out_of_bounds 0
	foreach { x y } $args {
	    $this AddOnePoint $curveid $x $y
	    if {$x<$xgraphmin || $x>$xgraphmax || \
		    $y<$ygraphmin || $y>$ygraphmax} {
		set out_of_bounds 1
	    }
	}
	if {$autodraw && ![$curve lookup $curveid,hidden]} {
	    if {$out_of_bounds} {
		$this Rescale
		$this DrawCurves
	    } else {
		$this PlotCurve $curveid
	    }
	}
	return $curveid
    }
    method PlotCurve { curveid } { graph curve curve_width} {
	$graph delete tag$curveid
	if {![$curve exists $curveid,ptsdisp]} { return }
	if {![$curve lookup $curveid,scalevalid]} {
	    $this RescaleCurve $curveid
	}
	set ptlist [$curve lookup $curveid,ptsdisp]
	set ptcount [llength $ptlist]
	if {$ptcount>=4} {
	    eval $graph create line $ptlist \
		    -fill [$curve lookup $curveid,color] \
		    -width $curve_width -tags tag$curveid
	}
	return $ptcount	
    }
    method DrawCurves {} { id curve curvecount } {
	# Note: We want to draw curves in order
	set curves_drawn 0
	for { set i 0 } { $i < $curvecount } { incr i } {
	    set curveid $i
	    if {[$curve exists $curveid,hidden] && \
		    ![$curve lookup $curveid,hidden] } {
		# Curve exists and is not hidden
		$this PlotCurve $curveid
		incr curves_drawn
	    }
	}
	return $curves_drawn
    }
    method SetXLabel { newxlabel } { xlabel } {
	set xlabel $newxlabel
    }
    method SetYLabel { newylabel } { ylabel } {
	# Separate ylabel characters by newlines
	set ylabel [join [split $newylabel {}] "\n"]
    }
    method SetLabels { newxlabel newylabel } {} {
	$this SetXLabel $newxlabel
	$this SetYLabel $newylabel
	$this DrawAxes
    }
    method GetLabels {} { xlabel ylabel } {
	return [list $xlabel $ylabel]
    }
    method DrawAxes {} {
	$graph delete axes
	set canwidth [$graph cget -width]
	set canheight [$graph cget -height]
	set xdispmin $xmargin
	set xdispmax [expr ($xgraphmax-$xorg)*$xscale+$xmargin]
	set ydispmin $ymargin
	set ydispmax [expr ($ygraphmin-$yorg)*$yscale+$ymargin]
	$graph create line \
		$xdispmin $ydispmin \
		$xdispmin $ydispmax \
		$xdispmax $ydispmax \
		$xdispmax $ydispmin \
		$xdispmin $ydispmin \
		-fill $axescolor -width $axeswidth -tags axes
	$graph create text $xdispmin [expr $ydispmax+$ymargin*0.1] \
		-text $xgraphmin -anchor n -tags axes
	$graph create text $xdispmax [expr $ydispmax+$ymargin*0.1] \
		-text $xgraphmax -anchor n -tags axes
	$graph create text [expr $xdispmin-$xmargin*0.1] $ydispmax \
		-text $ygraphmin -anchor se -tags axes
	$graph create text [expr $xdispmin-$xmargin*0.1] $ydispmin \
		-text $ygraphmax -anchor ne -tags axes

	# Place labels
	$graph create text \
		[expr ($xdispmin+$xdispmax)/2.] \
		[expr $canheight-$ymargin*0.1]  \
		-text $xlabel -anchor s -tags axes
	$graph create text \
		[expr $xmargin*0.2]             \
		[expr ($ydispmin+$ydispmax)/2.] \
		-text $ylabel -anchor w -justify center -tags axes
	
	# X-axes tics
	set xquart [expr ($xdispmax-$xdispmin)/4.]
	set x1 [expr $xdispmin+$xquart]
	set y1 $ydispmax  ;  set y2 [expr $y1-1.4*$axeswidth]
	set y3 $ydispmin  ;  set y4 [expr $y3+1.4*$axeswidth]
	$graph create line $x1 $y1 $x1 $y2 \
		-fill $axescolor -width $axeswidth -tags axes
	$graph create line $x1 $y3 $x1 $y4 \
		-fill $axescolor -width $axeswidth -tags axes
	set x1 [expr $xdispmax-$xquart]
	$graph create line $x1 $y1 $x1 $y2 \
		-fill $axescolor -width $axeswidth -tags axes
	$graph create line $x1 $y3 $x1 $y4 \
		-fill $axescolor -width $axeswidth -tags axes
	set x1 [expr $xdispmin+2*$xquart]
	set y1 $ydispmax  ;  set y2 [expr $y1-2*$axeswidth]
	set y3 $ydispmin  ;  set y4 [expr $y3+2*$axeswidth]
	$graph create line $x1 $y1 $x1 $y2 \
		-fill $axescolor -width $axeswidth -tags axes
	$graph create line $x1 $y3 $x1 $y4 \
		-fill $axescolor -width $axeswidth -tags axes
	# Y-axes tics
	set yquart [expr ($ydispmax-$ydispmin)/4.]
	set y1 [expr $ydispmax-$yquart]
	set x1 $xdispmin  ;  set x2 [expr $x1+1.4*$axeswidth]
	set x3 $xdispmax  ;  set x4 [expr $x3-1.4*$axeswidth]
	$graph create line $x1 $y1 $x2 $y1 \
		-fill $axescolor -width $axeswidth -tags axes
	$graph create line $x3 $y1 $x4 $y1 \
		-fill $axescolor -width $axeswidth -tags axes
	set y1 [expr $ydispmin+$yquart]
	$graph create line $x1 $y1 $x2 $y1 \
		-fill $axescolor -width $axeswidth -tags axes
	$graph create line $x3 $y1 $x4 $y1 \
		-fill $axescolor -width $axeswidth -tags axes
	set y1 [expr $ydispmax-2*$yquart]
	set x1 $xdispmin  ;  set x2 [expr $x1+2*$axeswidth]
	set x3 $xdispmax  ;  set x4 [expr $x3-2*$axeswidth]
	$graph create line $x1 $y1 $x2 $y1 \
		-fill $axescolor -width $axeswidth -tags axes
	$graph create line $x3 $y1 $x4 $y1 \
		-fill $axescolor -width $axeswidth -tags axes
    }
    private method RescaleCurve { curveid } \
	    { curve xorg yorg xscale yscale xmargin ymargin } {
	# Rescale all display points
	if {![$curve exists $curveid,pts]} { return }
	set xoffset [expr $xmargin-$xorg*$xscale]
	set yoffset [expr $ymargin-$yorg*$yscale]
	set newlist {}
	$curve set [list $curveid,ptsdisp {}]
	foreach { x y } [$curve lookup $curveid,pts] {
	    set xdisp [expr $x*$xscale+$xoffset]
	    set ydisp [expr $y*$yscale+$yoffset]
	    append newlist " $xdisp $ydisp"
	}
	$curve set [list $curveid,ptsdisp $newlist]
	$curve set [list $curveid,scalevalid 1]
    }
    method Rescale { {use_data 1} } {
	# Set import use_data to non-zero to invoke data-based
	# graph extents changes.  Either way, rescales the
	# graph as necessary to fill the canvas.
	if {$use_data} {
	    # Find data extents
	    for { set cid 0 } { $cid < $curvecount } { incr cid } {
		if {![$curve exists $cid,xmin] || \
			[$curve lookup $cid,hidden] } {
		    continue     ;# Empty or hidden curve
		}
		if {![info exists xmin]} {
		    # First non-empty curve
		    set xmin [$curve lookup $cid,xmin]
		    set xmax [$curve lookup $cid,xmax]
		    set ymin [$curve lookup $cid,ymin]
		    set ymax [$curve lookup $cid,ymax]
		} else {
		    # Normal case
		    set test [$curve lookup $cid,xmin]
		    if { $test < $xmin } { set xmin $test }
		    set test [$curve lookup $cid,xmax]
		    if { $test > $xmax } { set xmax $test }
		    set test [$curve lookup $cid,ymin]
		    if { $test < $ymin } { set ymin $test }
		    set test [$curve lookup $cid,ymax]
		    if { $test > $ymax } { set ymax $test }
		}
	    }
	    if {![info exists xmin]} { 
		# No data, so fill in dummy values
		set xmin 0  ; set ymin 0
		set xmax 1  ; set ymax 1
	    }
	    if { $xmin >= $xmax } { set xmax [expr $xmin +1] } ;# Safety
	    if { $ymin >= $ymax } { set ymax [expr $ymin +1] }

	    # Adjust to "round" numbers
	    set xdelta [expr $xmax-$xmin]
	    if {$xdelta<=0.0} {set xdelta 1. } ;# Safety 
	    set xrangescale [expr pow(10,ceil(-log10($xdelta)))]
	    set xmin [expr (floor($xmin*$xrangescale))/$xrangescale]
	    set xmax [expr (ceil($xmax*$xrangescale))/$xrangescale]
	    set ydelta [expr $ymax-$ymin]
	    if {$ydelta<=0.0} {set ydelta 1. } ;# Safety 
	    set yrangescale [expr pow(10,ceil(-log10($ydelta)))]
	    set ymin [expr (floor($ymin*$yrangescale))/$yrangescale]
	    set ymax [expr (ceil($ymax*$yrangescale))/$yrangescale]

	    # Round and save for future use
	    set xgraphmin [format "%g" $xmin]
	    set xgraphmax [format "%g" $xmax]
	    if { $xgraphmax<=$xgraphmin } {
		# Too much rounding.  Try again
		set xgraphmin [format "%.15g" $xmin]
		set xgraphmax [format "%.15g" $xmax]
	    }
	    set ygraphmin [format "%g" $ymin]
	    set ygraphmax [format "%g" $ymax]
	    if { $ygraphmax<=$ygraphmin } {
		# Too much rounding.  Try again
		set ygraphmin [format "%.15g" $ymin]
		set ygraphmax [format "%.15g" $ymax]
	    }
	} ;# End of if {$use_data}

 	if {$xgraphmax<=$xgraphmin} {
	    # Safety.  This hopefully won't happen
	    set xgraphmax [expr $xgraphmin + 1. ]
	}
 	if {$ygraphmax<=$ygraphmin} {
	    # Safety.  This hopefully won't happen
	    set ygraphmax [expr $ygraphmin + 1. ]
	}

	# Calculate lower bound for margin size from axes coordinate text
	$graph delete margin_test   ;# Safety
	$graph create text 0 0 -text $xgraphmin -anchor sw -tags margin_test
	$graph create text 0 0 -text $xgraphmax -anchor sw -tags margin_test
	set bbox [$graph bbox margin_test]
	set xtext_height [expr ([lindex $bbox 3] - [lindex $bbox 1])*1.1+1]
	## Include a little bit extra
	$graph delete margin_test
	$graph create text 0 0 -text $ygraphmin -anchor sw -tags margin_test
	$graph create text 0 0 -text $ygraphmax -anchor sw -tags margin_test
	set bbox [$graph bbox margin_test]
	set ytext_width [expr ([lindex $bbox 2] - [lindex $bbox 0])*1.1+1]
	## Include a little bit extra
	$graph delete margin_test
	
	# Set origin and scaling factors
	# NOTE: Canvas widget has +y downward
	set canwidth [$graph cget -width]
	set canheight [$graph cget -height]
	set xorg_new $xgraphmin
	set yorg_new $ygraphmax
	set xmargin_new [expr $canwidth*(1-$graphsize)/2.]
	set ymargin_new [expr $canheight*(1-$graphsize)/2.]
	if { $xmargin_new < $ytext_width }  { set xmargin_new $ytext_width }
	if { $ymargin_new < $xtext_height } { set ymargin_new $xtext_height }
	set xscale_new  [expr ($canwidth-2*$xmargin_new)/($xgraphmax-$xgraphmin)]
	set yscale_new  [expr ($canheight-2*$ymargin_new)/($ygraphmin-$ygraphmax)]

	# See if any changes were made
	if {[info exists xorg] && $xorg_new==$xorg && $yorg_new==$yorg && \
		$xmargin_new==$xmargin && $ymargin_new==$ymargin && \
		$xscale_new==$xscale && $yscale_new==$yscale} {
	    # No changes, so no need to rescale data or redraw axes
	    return
	}
	set xorg    $xorg_new        ; set yorg    $yorg_new
	set xmargin $xmargin_new     ; set ymargin $ymargin_new
	set xscale  $xscale_new      ; set yscale  $yscale_new

	# Rescale display points on all non-hidden curves
	for { set cid 0 } { $cid < $curvecount } { incr cid } {
	    if {[$curve exists $cid,pts]} {
		if {[$curve lookup $cid,hidden]} {
		    $curve set [list $cid,scalevalid 0]
		    ## Mark scaling invalid
		} else {
		    $this RescaleCurve $cid
		}
	    }
	}
	# Delete current graph and redraw
	$graph delete all
	$this DrawAxes
	$this DrawCurves
    }
    method PrintData { curvename } {
	# For Debugging
	if {![$id exists $curvename]} {
	    # No match
	    return
	}
	set curveid [$id lookup $curvename]
	foreach i [lsort -ascii [$curve names $curveid,*]] {
	    puts "$i = [$curve lookup $i]"
	}
	return
    }
    method HasGui {} {
        return 0
    }
    method WinDestroy { w } { winpath } {
	if {[string compare $winpath $w]!=0} {
	    return    ;# Child destroy event
	}
	$this delete
    }
    destructor {
	# Notify client of impending destruction
	eval $callback $this DELETE {$delete_callback_arg}
        em_eventHandler Generate delete $bindtags

	catch { $color delete }
	catch { $id delete }
	catch { $curve delete }
	if {[info exists winpath] && [winfo exists $winpath]} {
            # Remove <Destroy> binding
	    bind $winpath <Destroy> {}
	    # Destroy instance frame, all children, and bindings
	    destroy $winpath
	}
    }
}

