if {$argc>0} { set mif_file [lindex $argv 0]} else {
    error "Usage: simplecontrol.tcl <mif-file>"
}

# Text output subroutines
proc PrintState { solver } {
    set stepcount [$solver GetODEStepCount]
    set torque [$solver GetMaxTorque]
    set maxangle [$solver GetMaxAngle]
    set energies [join [$solver GetEnergies]]
    puts [format "%5d %10.7f  %5.1f  %12.8f %12.8f %12.8f %12.8f %12.8f" \
	    $stepcount $torque $maxangle \
	    [lindex $energies 1] [lindex $energies 3] \
	    [lindex $energies 5] [lindex $energies 7] \
	    [lindex $energies 9]]
}

proc PrintLabels {} {
puts "  ODE     Max       Max                               ENERGIES"
puts " Step    Torque    Angle     Total       Exchange    Anisotropy     Demag        Zeeman"
}

# Graph update subroutine
proc InspectLoop { solver graph } {
    global last_step
    set this_step [$solver GetODEStepCount]
    if {![info exists last_step] || $last_step!=$this_step} {
	set torque [$solver GetMaxTorque]
	set energies [join [$solver GetEnergies]]
	if {1 || $this_step % 10 == 0} {
	    puts "Step [format "%4d" $this_step]: \
		    Max Torque=[format "%7.5f" $torque]"
	}
	$graph AddPoints "Total Energy" $this_step [lindex $energies 1]
	$graph AddPoints "Exchange Energy" $this_step [lindex $energies 3]
	$graph AddPoints "Anisotropy Energy" $this_step [lindex $energies 5]
	$graph AddPoints "Demag Energy" $this_step [lindex $energies 7]
	$graph AddPoints "Zeeman Energy" $this_step [lindex $energies 9]
    }
    set last_step $this_step
    after 20 InspectLoop $solver $graph
}

# Initialize solver and display graph
set mif [mms_mif new]
$mif Read $mif_file
set solver [mms_solver new $mif]
$solver configure -relax_stop_torque 1e-5

source [file join [file dirname [info script]] graphbox.tcl]

# Initialize display graph
set graph [GraphBox new . -exit_button 0 -autodraw 1 -curve_width 3]
$graph SetLabels "Iteration" "Energy"
$graph NewCurve "Total Energy"
$graph NewCurve "Exchange Energy"
$graph NewCurve "Anisotropy Energy"
$graph NewCurve "Demag Energy"
$graph NewCurve "Zeeman Energy"
pack [$graph cget -winpath] -side right -fill both -expand 1

# Build control buttons
set btn_frame [frame .f]
set b0 [button .b0 -text Labels -command PrintLabels]
set b1 [button .b1 -text "Run" \
	-command "InspectLoop $solver $graph ; $solver RelaxState"]
set b2 [button .b2 -text "Print State" \
	-command "PrintState $solver"]
set b3 [button .b3 -text "Halt" \
	-command "$solver configure -stop_request_flag USER ; \
                  after cancel InspectLoop $solver $graph"]
set b4 [button .b4 -text "Exit" \
	-command "$solver delete ; exit"]
pack $b0 $b1 $b2 $b3 $b4 -in $btn_frame -side top -fill both -expand 1
pack $btn_frame -side left -fill y
