# FILE: procs.tcl
#
# A collection of Tcl procedures used by pimake application

set oommfExtDir [file join [file dirname [file dirname [file dirname \
                [Oc_DirectPathname [info script]]]]] ext]

proc DeleteFiles {args} {
    if {![llength $args]} {return}
    puts "Deleting $args in [pwd] ..."
    eval file delete -force -- $args
}

proc MakeDirectory {dir} {
    puts "Making directory '[file join [pwd] $dir]' ..."
    file mkdir $dir
}

proc MakeTclIndex {dir args} {
    puts "Updating [file join [pwd] tclIndex] ..."
    eval auto_mkindex $dir $args
    global errorCode errorInfo
    set oldDir [pwd]
    cd $dir
    set dir [pwd]
    append index "# The following lines were appended to this file by the\n"
    append index "# command 'MakeTclIndex' of the pimake application.  They\n"
    append index "# provide entries in the auto_index array to support the\n"
    append index "# auto-loading of Oc_Classes.\n\n"
    if {$args == ""} {
	set args *.tcl
    }
    foreach file [eval glob $args] {
	set f ""
	set error [catch {
	    set f [open $file]
	    while {[gets $f line] >= 0} {
		if [regexp {^Oc_Class[ 	]+([^ 	]*)} $line match className] {
		    append index "set [list auto_index($className)]"
                    # Should this be at global scope?
		    append index " \[list uplevel #0 source \[file join \$dir [list $file]\]\]\n"
		}
	    }
	    close $f
	} msg]
	if $error {
	    set code $errorCode
	    set info $errorInfo
	    catch {close $f}
	    cd $oldDir
	    error $msg $info $code
	}
    }
    set f ""
    set error [catch {
	set f [open tclIndex a]
	puts $f $index nonewline
	close $f
	cd $oldDir
    } msg]
    if $error {
	set code $errorCode
	set info $errorInfo
	catch {close $f}
	cd $oldDir
	error $msg $info $code
    }
}

proc Recursive {target} {
    foreach mr [glob -nocomplain [file join * makerules.tcl]] {
        if {[MakeRule SourceRulesFile $mr]} {
            MakeRule Build [file join [file dirname \
                    [Oc_DirectPathname $mr]] $target]
        }
    }
}

# A wrapper for the Tcl command 'exec' which also informs the user 
# what it is doing.
# Should probably modify to use Oc_Log or some such mechanism.
proc Exec {args} {
    global errorCode errorInfo
    puts [join $args]
    if {[catch {eval exec $args} res]} {
        set ec $errorCode
        set ei $errorInfo
        if {[string match CHILDSTATUS [lindex $ec 0]] && [lindex $ec 2]} {
            error $res $ei $ec
        }
        puts $res
    }
    return $res
}

