# FILE: wintel.tcl
#
# Configuration feature definitions for the configuration 'wintel'
#
# Editing instructions begin at "START EDIT HERE" below.

set config [Oc_Config RunPlatform]

if {![string match [file rootname [file tail [info script]]] \
        [$config GetValue platform_name]]} {
    error "Configuration cache file '[info script]' 
sourced by '[$config GetValue platform_name]'"
}

########################################################################
# START EDIT HERE
# In order to properly build, install, and run on your computing 
# platform, the OOMMF software must know certain features of your
# computing environment.  In this file are lines which set the value of 
# certain features of your computing environment.  Each line looks like:
#
# $config SetValue <feature> {<value>}
#
# where each <feature> is the name of some feature of interest,
# and <value> is the value which is assigned to that feature in a
# description of your computing environment.  Your task is to edit
# the values as necessary to properly describe your computing 
# environment.
#
# The character '#' at the beginning of a line is a comment character.
# It causes the contents of that line to be ignored.  To select
# among lines providing alternative values for a feature, uncomment the
# line containing the proper value.
#
# The features in this file are divided into three sections.  The first
# section (REQUIRED CONFIGURATION) includes features which require you 
# to provide a value.  The second section (OPTIONAL CONFIGURATION)
# includes features which have usable default values, but which you
# may wish to customize.  The third section (ADVANCED CONFIGURATION)
# contains features which you probably do not need or want to change
# without a good reason.
########################################################################
# REQUIRED CONFIGURATION

# Set the feature 'path_directory_temporary' to the name of an existing 
# directory on your computer in which OOMMF software should write 
# temporary files.  All OOMMF users must have write access to this 
# directory.
#
$config SetValue path_directory_temporary {C:\temp}
# $config SetValue path_directory_temporary {C:\}

# NOTE: The rest of the REQUIRED CONFIGURATION is required only
# for building OOMMF software from source code.  If you dowloaded
# a distribution with pre-compiled executables, no more configuration
# is required.

# Set the feature 'program_compiler_c++' to the program to run on this 
# platform to compile source code files written in the language C++ into 
# object files.  Select from the choices below.  If the compiler is not 
# in your path, be sure to use the whole pathname.  Also include any 
# options required to instruct your compiler to only compile, not link.  
#
# If your compiler is not listed below, additional features will
# have to be added in the ADVANCED CONFIGURATION section below to 
# describe to the OOMMF software how to operate your compiler.  Send
# e-mail to the OOMMF developers for assistance.
#
# Microsoft Visual C++
# <URL:http://www.microsoft.com/visualc/>
$config SetValue program_compiler_c++ {cl /c}

########################################################################
# OPTIONAL CONFIGURATION

########################################################################
# ADVANCED CONFIGURATION

# Major version of Tcl you have installed
$config SetValue TCL_MAJOR_VERSION [lindex [split [info tclversion] .] 0]

# Minor version of Tcl you have installed
$config SetValue TCL_MINOR_VERSION [lindex [split [info tclversion] .] 1]

if {[info exists tk_version]} {
    # Major version of Tk you have installed
    $config SetValue TK_MAJOR_VERSION [lindex [split $tk_version .] 0]

    # Minor version of Tk you have installed
    $config SetValue TK_MINOR_VERSION [lindex [split $tk_version .] 1]
} else {
    if {[info tclversion] < 8} {
        set tk_version [expr [info tclversion] - 3.4]
    } else {
        set tk_version [info tclversion]
    }
    # Major version of Tk you have installed
    $config SetValue TK_MAJOR_VERSION [lindex [split $tk_version .] 0]

    # Minor version of Tk you have installed
    $config SetValue TK_MINOR_VERSION [lindex [split $tk_version .] 1]
    unset tk_version
}

# Compiler option processing...
if {[string match cl [file tail [lindex \
        [$config GetValue program_compiler_c++] 0]]]} {
    # ...for Microsoft Visual C++
    $config SetValue program_compiler_c++_option_opt {}
    $config SetValue program_compiler_c++_option_out {format "\"/Fo%s\""}
    $config SetValue program_compiler_c++_option_src {format "\"/Tp%s\""}
    $config SetValue program_compiler_c++_option_inc {format "\"/I%s\""}
}

# The program to run on this platform to link together object files and
# library files to create an executable binary.
#
if {[string match cl [file tail [lindex \
        [$config GetValue program_compiler_c++] 0]]]} {
    # Microsoft Visual C++'s linker
    $config SetValue program_linker {link}
}

# Linker option processing...
if {[string match link [file tail [lindex \
        [$config GetValue program_linker] 0]]]} {
    # ...for Microsoft VC++ linker
    $config SetValue program_linker_option_obj {format \"%s\"}
    $config SetValue program_linker_option_out {format "\"/OUT:%s\""}
    $config SetValue program_linker_option_lib {format \"%s\"}
    $config SetValue program_linker_option_sub {format "\"/SUBSYSTEM:%s\""}
    if {[$config GetValue TK_MAJOR_VERSION] < 8} {
        $config SetValue TK_LIB_SPEC [list [file join [file dirname \
                [info library]] tk[$config GetValue TK_MAJOR_VERSION][$config \
                GetValue TK_MINOR_VERSION].lib]]
    } else {
        $config SetValue TK_LIB_SPEC [list [file join [file dirname \
                [info library]] tk[$config GetValue TK_MAJOR_VERSION][$config \
                GetValue TK_MINOR_VERSION]vc.lib]]
    }
    if {[$config GetValue TCL_MAJOR_VERSION] < 8} {
        $config SetValue TCL_LIB_SPEC [list [file join [file dirname \
                [info library]] tcl[$config GetValue \
                TCL_MAJOR_VERSION][$config GetValue TCL_MINOR_VERSION].lib]]
    } else {
        $config SetValue TCL_LIB_SPEC [list [file join [file dirname \
                [info library]] tcl[$config GetValue \
                TCL_MAJOR_VERSION][$config GetValue TCL_MINOR_VERSION]vc.lib]]
    }
    $config SetValue TK_LIBS {}
    $config SetValue TCL_LIBS {}
    $config SetValue program_linker_uses_-L-l {0}
}

# The program to run on this platform to create a single library file out
# of many object files.
if {[string match cl [file tail [lindex \
        [$config GetValue program_compiler_c++] 0]]]} {
    # Microsoft Visual C++'s library maker
    $config SetValue program_libmaker {lib}
}

# Library making option processing...
if {[string match lib [file tail [lindex \
        [$config GetValue program_libmaker] 0]]]} {
    # ...for Microsoft VC++ lib
    $config SetValue program_libmaker_option_obj {format \"%s\"}
    $config SetValue program_libmaker_option_out {format "\"/OUT:%s\""}
}

# The absolute, native filename of the null device
$config SetValue path_device_null {nul:}

# A partial Tcl command (or script) which when completed by lappending
# a file name stem and evaluated returns the corresponding file name for
# an executable on this platform
$config SetValue script_filename_executable {format %s.exe}

# A partial Tcl command (or script) which when completed by lappending
# a file name stem and evaluated returns the corresponding file name for
# an object file on this platform
$config SetValue script_filename_object {format %s.obj}

# A partial Tcl command (or script) which when completed by lappending
# a file name stem and evaluated returns the corresponding file name for
# a static library on this platform
$config SetValue script_filename_static_library {format %s.lib}

# Installation location of Tcl/Tk
#	Use the Windows registry if it is available
if {![catch {package require registry}]  && ![catch {registry get \
        HKEY_LOCAL_MACHINE\\SOFTWARE\\Sun\\Tcl\\[info tclversion] Root
        } result]} {
    $config SetValue TCL_EXEC_PREFIX $result
} else {
    # Can't access registry -- try backup plan
    $config SetValue TCL_EXEC_PREFIX [file dirname [file dirname \
            [Oc_DirectPathname [info library]]]]
}

# The absolute, native filename of a directory containing
# the X11 include files
$config SetValue TK_XINCLUDES \
        [file join [$config GetValue TCL_EXEC_PREFIX] include X11]

########################################################################

