# FILE: makerules.tcl
#
# This file controls the application 'pimake' by defining rules which
# describe how to build the applications and/or extensions produced by
# the source code in this directory.
#
# NOTICE: Please see the file ../../LICENSE
#
# Last modified on: $Date: 1998/07/14 19:18:45 $
# Last modified by: $Author: dgp $
#
# Verify that this script is being sourced by pimake
if {[llength [info commands MakeRule]] == 0} {
    error "'[info script]' must be evaluated by pimake"
}
#
########################################################################

set objects {
	dstring
       	evoc
	functions
	nb
	stopwatch
}

MakeRule Define {
    -targets            build
    -dependencies       [concat configure tclIndex [Platform StaticLibrary nb]]
}

MakeRule Define {
    -targets            configure
    -dependencies       [Platform Name]
}

MakeRule Define {
    -targets            [Platform Name]
    -dependencies       {}
    -script             {MakeDirectory [Platform Name]}
}

MakeRule Define {
    -targets            tclIndex
    -dependencies       [glob -nocomplain *.tcl]
    -script             {
                         MakeTclIndex .
                        }
}

MakeRule Define {
    -targets            [Platform StaticLibrary nb]
    -dependencies	[Platform Objects [concat errhandlers $objects]]
    -script		[format {
			    eval file delete [Platform StaticLibrary nb]
			    Platform MakeLibrary -out nb -obj {errhandlers %s}
			    Platform IndexLibrary nb
			} $objects]
}

foreach o $objects { 
    MakeRule Define {
        -targets	[Platform Objects $o]
        -dependencies	[concat [list [Platform Name]] \
			        [[CSourceFile New _ $o.cc] Dependencies]]
        -script		[format {Platform Compile C++ -opt 1 \
			        -inc [[CSourceFile New _ %s.cc] DepPath] \
			        -src %s.cc -out %s
			} $o $o $o]
    }
}

# On some configurations (platform-compiler combinations), there are C++ 
# compiler bugs which cause the combination of optimization and use of 
# variable-length argument lists in functions to lead to core dumps.  
# On those configurations, we disable optimization when compiling 
# errhandlers.cc (which contains functions with variable-length argument lists.
set optimizationLevel 1
if {![catch {[Oc_Config RunPlatform] GetValue \
        program_compiler_c++_property_optimization_breaks_varargs}]} {
    set optimizationLevel 0
}

MakeRule Define {
    -targets		[Platform Objects errhandlers]
    -dependencies	[concat [list [Platform Name]] [[CSourceFile New _ \
			        errhandlers.cc] Dependencies]]
    -script		[format {Platform Compile C++ -opt %s -inc \
			        [[CSourceFile New _ errhandlers.cc] DepPath] \
			        -src errhandlers.cc -out errhandlers
			} $optimizationLevel]
}

MakeRule Define {
    -targets            distclean
    -dependencies       clean
    -script             {DeleteFiles [Platform Name]}
}
 
MakeRule Define {
    -targets            clean
    -dependencies       mostlyclean
}
 
MakeRule Define {
    -targets            mostlyclean
    -dependencies       objclean
    -script             {eval DeleteFiles [concat tclIndex \
			        [Platform StaticLibrary nb]]}
}
 
MakeRule Define {
    -targets            objclean
    -dependencies       {}
    -script             [format {
			    eval DeleteFiles [Platform Objects {errhandlers %s}]
			} $objects]
}

unset objects
